module CUDD

# cudd_path = joinpath(dirname(@__FILE__), "..", "deps", "cudd-3.0.0", "cudd", ".libs")
const depfile = joinpath(@__DIR__, "..", "deps", "deps.jl")

if isfile(depfile)
    include(depfile)
else
    error("CUDD not properly installed. Please run Pkg.build(\"CUDD\")")
end

using CEnum

# Files automatically generated by Clang.jl
include(joinpath(@__DIR__, "gen", "libcudd_common.jl"))
include(joinpath(@__DIR__, "gen", "libcudd_api.jl"))

# Missing C macros omitted by Clang.jl

Cudd_Not(node) =
    convert(Ptr{Nothing}, xor(convert(Int,node), 1))

Cudd_IsComplement(node) =
    isone(convert(Int,node) & 1)

Cudd_Regular(node) = 
    convert(Ptr{Nothing}, convert(Int,node) & ~1)

# export everything
foreach(names(@__MODULE__, all=true)) do s
    if startswith(string(s), "Cudd_")
        @eval export $s
    end
end

# More user-friendly interface for ADD
include("ADD.jl")
include("ADD_apply.jl")
include("ADD_find.jl")
include("ADD_abs.jl")
include("ADD_inverse.jl")
include("ADD_ITE.jl")
include("ADD_negate.jl")
include("ADD_walsh.jl")

end # module
